//+------------------------------------------------------------------+
//|                                                     Chimp2_1.mq4 |                                     
//+------------------------------------------------------------------+
//Revision history; 
//Chimp2_1 is a hybrid between earlier Chimp1_n and Chimp2_2 which uses Price/Volume instead of price.
//
#property indicator_separate_window
#property indicator_buffers   3
#property indicator_color1    Lime
#property indicator_color2    Red
#property indicator_color3    Aqua
#property indicator_width1    2
#property indicator_width2    2
#property indicator_width3    2

//---- input parameters
extern double  WeightDM       = 2.5;
extern double  WeightDI       = 2.0;
extern double  WeightDX       = 1.0;
extern double  WeightOut      = 1.0;

extern bool   alertsOn        = true;
extern bool   alertsOnCurrent = false;
extern bool   alertsMessage   = true;
extern bool   alertsSound     = true;
extern bool   alertsEmail     = false;
extern string soundfile       = "alert2.wav";


//---- buffers
double MA[];
double UP[];
double DN[];
double PDM[];//Not displayed
double MDM[];//Not displayed
double PDI[];//Not displayed
double MDI[];//Not displayed
double DX[];//Not displayed
double trend[];

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
{
//---- indicators
   IndicatorBuffers(8);
   SetIndexBuffer(0,UP); SetIndexStyle(0,DRAW_HISTOGRAM); SetIndexLabel(0,"Chimp2_1");
   SetIndexBuffer(1,DN); SetIndexStyle(1,DRAW_HISTOGRAM); SetIndexLabel(1,"Chimp2_1");
   SetIndexBuffer(2,MA);                                  SetIndexLabel(2,"Chimp2_1");
   SetIndexBuffer(3,PDM); 
   SetIndexBuffer(4,MDM);   
   SetIndexBuffer(5,PDI);
   SetIndexBuffer(6,MDI); 
   SetIndexBuffer(7,DX);
   
   //---- name for DataWindow and indicator subwindow label
   string short_name="Chimp2_1";
   IndicatorShortName(short_name);
   
   
   
   //----
   SetLevelStyle(STYLE_DOT,1,White);  
   SetLevelValue(0,0.6);
   SetLevelValue(1,0.3);
   SetLevelValue(2,-0.3);
   SetLevelValue(3,-0.6);  
    
//--------- Safety overrides on user settings
   if (WeightDM<1.0)WeightDM=1.0;
   if (WeightDI<1.0)WeightDI=1.0;
   if (WeightDX<1.0)WeightDX=1.0;
   if (WeightOut<1.0)WeightOut=1.0;
      
   return(0);
}

//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
{  
   int i,r,counted_bars  = IndicatorCounted(); 
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
          int limit = MathMin(Bars - counted_bars,Bars-1);
          if (ArrayRange(trend,0)!=Bars) ArrayResize(trend,Bars);

   //
   //
   //
   //
   //
   
   for(i=limit, r=Bars-i-1; i>=0; i--,r++)
   { 
   PDM[i]=0;
   MDM[i]=0;
   if(Close[i]>Close[i+1])PDM[i]=Close[i]  - Close[i+1];
   else                   MDM[i]=Close[i+1]- Close[i];
   PDM[i]*=PDM[i];
   PDM[i]/=Volume[i];
   MDM[i]*=MDM[i];
   MDM[i]/=Volume[i];    
   PDM[i]=((WeightDM-1)*PDM[i+1] + PDM[i])/WeightDM;//ema.
   MDM[i]=((WeightDM-1)*MDM[i+1] + MDM[i])/WeightDM;//ema.
   
   double TR=PDM[i]+MDM[i];
   if (TR>0) { PDI[i]=PDM[i]/TR; MDI[i]=MDM[i]/TR; }//Avoid division by zero. Minimum step size is one unnormalized price pip.
   else      { PDI[i]=0;         MDI[i]=0; }
   
   PDI[i]=((WeightDI-1)*PDI[i+1] + PDI[i])/WeightDI;//ema.
   MDI[i]=((WeightDI-1)*MDI[i+1] + MDI[i])/WeightDI;//ema.
   double DI_Diff=PDI[i]-MDI[i];  
   if (DI_Diff<0) DI_Diff= -DI_Diff;//Only positive momentum signals are used.
   double DI_Sum=PDI[i]+MDI[i];
   double DI_Factor=0;//Zero case, DI_Diff will also be zero when DI_Sum is zero.
   if (DI_Sum>0) DX[i]=DI_Diff/DI_Sum;//Factional, near zero when PDM==MDM (horizonal), near 1 for laddering.
   else MA[i]=0;
   DX[i]=((WeightDX-1)*DX[i+1] + DX[i])/WeightDX;
                        
   if(MDI[i]>PDI[i])DX[i]=-DX[i];//Liable to cause sudden directional change if large WeightDX or WeightOut numbers used.
   MA[i]=((WeightOut-1)*MA[i+1]+DX[i])/WeightOut;
     
   UP[i]    = EMPTY_VALUE;
   DN[i]    = EMPTY_VALUE;
   trend[r] = trend[r-1];
   if(MA[i]>0)trend[r] =  1;
   if(MA[i]<0)trend[r] = -1;
   if(trend[r] == 1) UP[i]=MA[i]; 
   if(trend[r] ==-1) DN[i]=MA[i]; 
   
   }
   
   //
   //
   //
   //
   //
      
   if (alertsOn)
   {
     if (alertsOnCurrent)
          int whichBar = 0;
     else     whichBar = 1; whichBar = Bars-whichBar-1;
     if (trend[whichBar] != trend[whichBar-1])
     if (trend[whichBar] == 1)
           doAlert("Buy");
     else  doAlert("Sell");       
   }
return(0);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

void doAlert(string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
      if (previousAlert != doWhat || previousTime != Time[0]) {
          previousAlert  = doWhat;
          previousTime   = Time[0];

          //
          //
          //
          //
          //

          message =  StringConcatenate(Symbol(), Period()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," Chimp ",doWhat);
             if (alertsMessage) Alert(message);
             if (alertsEmail)   SendMail(StringConcatenate(Symbol(), Period()," Chimp "),message);
             if (alertsSound)   PlaySound(soundfile);
      }
}

